import aiomcrcon
from nonebot import logger
from mcqq_tool.utils import get_client, rcon_connect

from . import change_ip, change_port, change_password
from ...database.db import DB as db, server_list


async def change_rcon_status(server_name):
    """更新Rcon状态"""
    for server in server_list:
        if (client := get_client(server_name=server_name)) and server.server_name == server_name:
            # 获取该服务器的信息
            server = await db.get_server(server_name=server_name)
            # 开
            if server.rcon_msg or server.rcon_cmd and not client.rcon:
                client.rcon = aiomcrcon.Client(
                    server.rcon_ip,
                    server.rcon_port,
                    server.rcon_password
                )
                await rcon_connect(rcon_client=client.rcon, server_name=server_name)

            # 关：rcon消息关、rcon命令关、rcon状态开
            if not server.rcon_msg and not server.rcon_cmd and client.rcon:
                await client.rcon.close()
                logger.success(f"[MC_QQ]丨[Server:{server.server_name}] 的Rcon已关闭")



import aiomcrcon
import websockets
from typing import Optional, Union, List
from mcqq_tool.config import CLIENTS, Client
from mcqq_tool.utils import send_msg_to_qq, remove_client, rcon_connect, get_client
from nonebot import get_bot, logger
from nonebot.adapters.onebot.v11 import GroupMessageEvent, Bot
from nonebot_plugin_guild_patch import GuildMessageEvent

from ...database.db import server_list, MinecraftServer
from ...utils import process_msg_for_ws, process_msg_for_rcon, get_type_id


async def ws_client(websocket: websockets.WebSocketServerProtocol):
    """WebSocket"""
    try:
        server_name = websocket.request_headers["x-self-name"].encode('utf-8').decode('unicode_escape')
    except KeyError as e:
        # 服务器名为空
        logger.error(f"[MC_QQ]丨未获取到该服务器的名称，连接断开：{e}")
        await websocket.close(1008, "[MC_QQ]丨未获取到该服务器的名称，连接断开")
        return
    else:
        try:
            client = CLIENTS[server_name]
        except KeyError as e:
            # 准备加入客户端列表
            from ...database import DB as db
            rcon_client: Optional[aiomcrcon.Client] = None
            if server := await db.get_server(server_name=server_name):
                # rcon状态：rcon_msg rcon_cmd rcon_password不等于默认密码
                if (server.rcon_msg or server.rcon_cmd) and server.rcon_password != "change_password":
                    rcon_client = aiomcrcon.Client(server.rcon_ip, server.rcon_port, server.rcon_password)
                    await rcon_connect(rcon_client=rcon_client, server_name=server_name)
            CLIENTS[server_name] = Client(server_name=server_name, websocket=websocket, rcon=rcon_client)
            logger.success(f"[MC_QQ]丨[Server:{server_name}] 已连接至 WebSocket 服务器")

            try:
                async for message in websocket:
                    await send_msg_to_qq(bot=get_bot(), message=message)
            except websockets.WebSocketException as e:
                # 移除当前客户端
                await remove_client(server_name=server_name)
                logger.error(f"[MC_QQ]丨[Server:{server_name}] 的 WebSocket 连接已断开：{e}")
            else:
                if websocket.closed:
                    await remove_client(server_name=server_name)
                    logger.error(f"[MC_QQ]丨[Server:{server_name}] 的 WebSocket 连接已断开")
        else:
            logger.error(f"[MC_QQ]丨[Server:{server_name}] 重复连接，连接断开")
            await websocket.close(1008, f"[MC_QQ]丨[Server:{server_name}] 重复连接，连接断开")


async def send_msg_to_mc(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """发送消息到 MC"""
    # 处理来自QQ的消息
    if client_list := await get_clients(event=event):
        for client_server in client_list:
            client = CLIENTS.get(client_server.server_name)
            # 如果 服务器的rcon已连接 且 服务器切换为rcon发送消息
            if client.rcon and client_server.rcon_msg:
                text_msg, msgJson = await process_msg_for_rcon(bot=bot, event=event)
                try:
                    await client.rcon.send_cmd(msgJson)
                except aiomcrcon.ClientNotConnectedError:
                    logger.error(f"[MC_QQ]丨[Server:{client.server_name}] 的Rcon未连接，发送消息失败")
                else:
                    logger.success(f"[MC_QQ_Rcon]丨发送至 [server:{client.server_name}] 的消息 \"{text_msg}\"")

            elif client.websocket:
                text_msg, msgJson = await process_msg_for_ws(bot=bot, event=event)
                try:
                    await client.websocket.send(msgJson)
                except websockets.WebSocketException:
                    logger.error(f"[MC_QQ]丨发送至 [Server:{client.server_name}] 的过程中出现了错误")
                    await remove_client(client.server_name)
                else:
                    logger.success(f"[MC_QQ]丨发送至 [server:{client.server_name}] 的消息 \"{text_msg}\"")


async def send_cmd_to_mc(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent], cmd: str):
    """发送命令到 Minecraft"""
    if client_list := await get_clients(event=event):
        for client_server in client_list:
            client = CLIENTS.get(client_server.server_name)

            if client.rcon and client_server.rcon_cmd:
                # 获取命令
                try:
                    # 发送命令并获得返回消息
                    back_msg = await client.rcon.send_cmd(cmd)
                    # 机器人发送信息
                    await bot.send(event, message=back_msg[0])
                except aiomcrcon.ClientNotConnectedError:
                    # 连接关闭则移除客户端
                    await remove_client(client.server_name)
                    logger.error(f"[MC_QQ_Rcon]丨发送至 [Server:{client.server_name}] 的过程中出现了错误")
                else:
                    logger.success(f"[MC_QQ_Rcon]丨发送至 [server:{client.server_name}] 的命令 \"{cmd}\"")


async def get_clients(event: Union[GroupMessageEvent, GuildMessageEvent]) -> List[MinecraftServer]:
    """获取 服务器名、ws客户端、rcon连接"""
    res: List[MinecraftServer] = []
    for per_server in server_list:
        if client := get_client(per_server.server_name):
            if client.websocket and client.server_name == per_server.server_name:
                for per_group in per_server.all_group_list:
                    if await get_type_id(event) == per_group.type_id:
                        res.append(per_server)
    return res



from nonebot import on_message, on_command
from nonebot.adapters import Message
from nonebot.params import CommandArg
from nonebot.adapters.onebot.v11 import Bot, GroupMessageEvent
from nonebot_plugin_guild_patch import GuildMessageEvent
from typing import Union

from .data_source import send_msg_to_mc, send_cmd_to_mc
from ...utils import msg_rule, permission_check

mc_qq = on_message(priority=2, rule=msg_rule)

mc_qq_cmd = on_command("minecraft_command", aliases={"mcc"}, priority=1, rule=msg_rule, block=True)

mc_qq_cmd.handle()(permission_check)


# 收到消息时
@mc_qq.handle()
async def msg_first_receive(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    await send_msg_to_mc(bot=bot, event=event)


# 收到命令时
@mc_qq_cmd.handle()
async def cmd_first_receive(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent], args: Message = CommandArg()):
    await send_cmd_to_mc(bot=bot, event=event, cmd=args.extract_plain_text())



import json
from typing import Union

from mcqq_tool.utils import get_member_nickname
from nonebot.adapters.onebot.v11 import Message, GroupMessageEvent, Bot, GROUP_ADMIN, GROUP_OWNER, PrivateMessageEvent
from nonebot.exception import FinishedException
from nonebot.internal.permission import Permission
from nonebot.internal.rule import Rule
from nonebot.matcher import Matcher
from nonebot.params import ArgPlainText, CommandArg
from nonebot.permission import SUPERUSER
from nonebot_plugin_guild_patch import ChannelDestroyedNoticeEvent, GuildMessageEvent

from . import plugin_config
from .database.db import rule_group_list, rule_guild_list


def to_me():
    if plugin_config.mc_qq_to_me:
        from nonebot.rule import to_me

        return to_me()

    async def _to_me() -> bool:
        return True

    return Rule(_to_me)


async def _guild_admin(bot: Bot, event: GuildMessageEvent):
    roles = set(
        role["role_name"]
        for role in (
            await bot.get_guild_member_profile(
                guild_id=event.guild_id, user_id=event.user_id
            )
        )["roles"]
    )
    return bool(roles & set(plugin_config.mc_qq_guild_admin_roles))


GUILD_ADMIN: Permission = Permission(_guild_admin)


async def permission_check(
        bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent, PrivateMessageEvent]
):
    # from ..database import DB as db

    if isinstance(event, PrivateMessageEvent):
        if event.sub_type == "group":  # 不处理群临时会话
            raise FinishedException
        if await SUPERUSER(bot, event):
            return

    if isinstance(event, GroupMessageEvent):
        # if not await db.get_group_admin(event.group_id):
        #     return
        if await (GROUP_ADMIN | GROUP_OWNER | SUPERUSER)(bot, event):
            return
    elif isinstance(event, GuildMessageEvent):
        # if not await db.get_guild_admin(event.guild_id, event.channel_id):
        #     return
        if await (GUILD_ADMIN | SUPERUSER)(bot, event):
            return
    await bot.send(event, "权限不足，目前只有管理员才能使用")
    raise FinishedException


async def handle_server_name(
        matcher: Matcher,
        command_arg: Message = CommandArg(),
):
    server_name = command_arg.extract_plain_text().strip()
    if server_name:
        matcher.set_arg("server_name", command_arg)


async def server_name_check(
        matcher: Matcher,
        server_name: str = ArgPlainText("server_name"),
):
    server_name = server_name.strip()
    matcher.set_arg("server_name", Message(server_name))


async def get_type_id(event: Union[GroupMessageEvent, GuildMessageEvent, ChannelDestroyedNoticeEvent]):
    if isinstance(event, GuildMessageEvent) or isinstance(event, ChannelDestroyedNoticeEvent):
        from .database import DB as db
        return await db.get_guild_type_id(event.guild_id, event.channel_id)
    return event.group_id


async def msg_rule(event: Union[GroupMessageEvent, GuildMessageEvent]) -> bool:
    """Rule 消息规则"""
    if isinstance(event, GroupMessageEvent):
        return event.group_id in rule_group_list
    elif isinstance(event, GuildMessageEvent):
        return f"{event.guild_id}:{event.channel_id}" in rule_guild_list
    return False


async def process_msg_for_ws(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """处理发送至MC的信息"""
    # 获取昵称
    member_nickname = await get_member_nickname(bot, event, event.user_id)

    # 初始化消息
    text_msg = member_nickname + "说："

    # 初始化消息字典
    messageList = []

    # 发送群聊名称
    from .database import DB as db
    if isinstance(event, GroupMessageEvent) and (await db.get_group(group_id=event.group_id)).send_group_name:
        group_name = {'msgType': "group_name",
                      'msgData': (await bot.get_group_info(group_id=event.group_id))['group_name']}
        messageList.append(group_name)
    elif isinstance(event, GuildMessageEvent) and (
            await db.get_guild(guild_id=event.guild_id, channel_id=event.channel_id)).send_group_name:
        group_name = {'msgType': "group_name"}
        guild_name = (await bot.get_guild_meta_by_guest(guild_id=event.guild_id))['guild_name']
        for per_channel in (await bot.get_guild_channel_list(guild_id=event.guild_id, no_cache=True)):
            if str(event.channel_id) == per_channel['channel_id']:
                channel_name = per_channel['channel_name']
                group_name['msgData'] = f"{guild_name}丨{channel_name}"
                break
        messageList.append(group_name)

    # 将群成员昵称装入消息列表
    messageList.append({"msgType": "senderName", "msgData": member_nickname})

    for msg in event.message:
        per_msg = {'msgType': msg.type}
        # 文本
        if msg.type == "text":
            msgData = msg.data['text'].replace("\r", "").replace("\n", "\n * ")
            text_msg += msgData
        # 图片
        elif msg.type == "image":
            msgData = msg.data['url']
            text_msg += '[图片]'
        # 表情
        elif msg.type == "face":
            msgData = '[表情]'
            text_msg += '[表情]'
        # 语音
        elif msg.type == "record":
            msgData = '[语音]'
            text_msg += '[语音]'
        # 视频
        elif msg.type == "video":
            msgData = msg.data['url']
            text_msg += '[视频]'
        # @
        elif msg.type == "at":
            # 获取被@ 群/频道 昵称
            at_member_nickname = await get_member_nickname(bot, event, msg.data['qq'])
            msgData = f"@{at_member_nickname}"
            text_msg += msgData
        # share
        elif msg.type == "share":
            msgData = msg.data['url']
            text_msg += '[分享：' + msg.data['title'] + ']'
        # forward
        elif msg.type == "forward":
            # TODO 将合并转发消息拼接为字符串
            # 获取合并转发 await bot.get_forward_msg(message_id=event.message_id)
            msgData = '[合并转发]'
            text_msg = msgData
        else:
            msgData = msg.type
            text_msg += '[' + msg.type + '] '

        text_msg += " "

        # 装入消息数据
        per_msg['msgData'] = msgData
        # 放入消息列表
        messageList.append(per_msg)

    return text_msg, str({"message": messageList})


async def process_msg_for_rcon(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """消息处理"""
    # 获取昵称
    member_nickname = await get_member_nickname(bot, event, event.user_id)

    # 初始化日志消息
    text_msg = member_nickname + " 说："

    command_msg = "tellraw @p "

    message_list = [
        {"text": "[MC_QQ] ", "color": "yellow"},
    ]
    # 是否发送群聊名称
    from .database import DB as db
    # 群
    if isinstance(event, GroupMessageEvent) and (await db.get_group(group_id=event.group_id)).send_group_name:
        message_list.append(
            {"text": (await bot.get_group_info(group_id=event.group_id))['group_name'] + " ", "color": "aqua"})
    # 频道
    elif isinstance(event, GuildMessageEvent) and (
            await db.get_guild(guild_id=event.guild_id, channel_id=event.channel_id)).send_group_name:
        guild_name = (await bot.get_guild_meta_by_guest(guild_id=event.guild_id))['guild_name']
        for per_channel in (await bot.get_guild_channel_list(guild_id=event.guild_id, no_cache=True)):
            if str(event.channel_id) == per_channel['channel_id']:
                message_list.append({"text": guild_name + "丨" + per_channel['channel_name'] + " ", "color": "aqua"})
                break
    message_list.append({"text": member_nickname, "color": "aqua"})
    message_list.append({"text": " 说：", "color": "yellow"})

    for msg in event.message:
        # 文本
        if msg.type == "text":
            msg_dict = {"text": msg.data['text'].replace("\r", "").replace("\n", "\n * ") + " ", "color": "white"}
            text_msg += msg.data['text'].replace("\r", "").replace("\n", "\n * ")
        # 图片
        elif msg.type == "image":
            msg_dict = {"text": "[图片] ", "color": "yellow",
                        "clickEvent": {"action": "open_url", "value": msg.data['url']},
                        "hoverEvent": {"action": "show_text", "contents": [{"text": "查看图片", "color": "gold"}]}
                        }
            text_msg += '[图片]'
        # 表情
        elif msg.type == "face":
            msg_dict = {"text": "[表情] ", "color": "gold"}
            text_msg += '[表情]'
        # 语音
        elif msg.type == "record":
            msg_dict = {"text": "[语音] ", "color": "light_purple"}
            text_msg += '[语音]'
        # 视频
        elif msg.type == "video":
            msg_dict = {"text": "[视频] ", "color": "light_purple",
                        "clickEvent": {"action": "open_url", "value": msg.data['url']},
                        "hoverEvent": {"action": "show_text", "contents": [{"text": "查看视频", "color": "dark_purple"}]}
                        }
            text_msg += '[视频]'
        # @
        elif msg.type == "at":
            # 获取被@ 群/频道 昵称
            at_member_nickname = await get_member_nickname(bot, event, msg.data['qq'])
            msg_dict = {"text": "@" + at_member_nickname + " ", "color": "green"}
            text_msg += f"@{at_member_nickname}"
        # share
        elif msg.type == "share":
            msg_dict = {"text": "[分享：" + msg.data['title'] + "] ", "color": "yellow",
                        "clickEvent": {"action": "open_url", "value": msg.data['url']},
                        "hoverEvent": {"action": "show_text", "contents": [{"text": "查看图片", "color": "gold"}]}
                        }
            text_msg += '[分享：' + msg.data['title'] + ']'
        # forward
        elif msg.type == "forward":
            # TODO 将合并转发消息拼接为字符串
            # 获取合并转发 await bot.get_forward_msg(message_id=event.message_id)
            msg_dict = {"text": "[合并转发] ", "color": "white"}
            text_msg += '[合并转发]'
        else:
            msg_dict = {"text": "[ " + msg.type + "] ", "color": "white"}
            text_msg += '[' + msg.type + ']'

        # 放入消息列表
        message_list.append(msg_dict)

    # 拼接完整命令
    command_msg += json.dumps(message_list)
    return text_msg, command_msg



from nonebot import on_command
from nonebot.adapters.onebot.v11 import MessageEvent

from mcqq_tool.utils import CLIENTS
from ..utils import permission_check, to_me

connected_servers = on_command("已连接服务器列表", rule=to_me(), priority=3)
connected_servers.__doc__ = """已连接服务器列表（管理员）"""

connected_servers.handle()(permission_check)


@connected_servers.handle()
async def _(event: MessageEvent):
    """发送所有已连接至 WebSocket 的服务器"""
    message = "已连接至 WebSocket 的服务器列表\n\n"

    for server_name, server in CLIENTS.items():
        message += (
            f"{server_name} "
            f"WebSocket: {'开' if server.websocket else '关'}, "
            f"Rcon: {'开' if server.rcon else '关'}\n"
        )
    await connected_servers.finish(message=message)
